/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** groupchat
*/

#include "common.h"


void cmd_join(char *arg)
{
	iks *x;
	iksid *id;
	textbuf *tb;

	id = iks_id_new(NULL, arg);
	if(!id || !id->user || !id->server)
	{
		gui_attr(GA_NOTICE);
		gui_print("Channel jid is invalid.\n");
              if (!options.compact_lines)
                 gui_print ("\n");
		return;
	}
	if(!id->resource) id->resource = net.id->user;

	tb = ui_buffer_new();
	tb->type = TB_GCHAT;
	tb->id = id;
	ui_buffer_show(tb);

	ui_print(tb, "Joining %s@%s...\n", id->user, id->server);

	x = iks_make_pres(IKS_TYPE_SET, IKS_SHOW_AVAILABLE, iks_id_print(id), NULL);
	iks_send(net.parser, x);
	iks_delete(x);

}


void cmd_nick(char *arg)
{
	iks *x = NULL;
        iksid *new = NULL;
        iksid *old = NULL;

	if(curbuf->type != TB_GCHAT) 
        {
                 goto usage;
        }

        if(!arg || *arg == '\0') {
                  goto usage;
        }

        new = iks_id_new(NULL, iks_id_printx(curbuf->id, IKS_ID_USER | IKS_ID_SERVER));
        new->resource = strdup(arg);

	x = iks_make_pres(IKS_TYPE_SET, IKS_SHOW_AVAILABLE, iks_id_print(new), NULL);
	iks_send(net.parser, x);
        iks_delete(x);

        old = curbuf->id;
        curbuf->id = new;
        /* FIXME: delete old id */

        return;

 usage:
        gui_attr(GA_NOTICE);
        gui_print("Usage: nick <new-nick>\n");
}

void cmd_part(char *arg)
{
	iks *x;

	if(curbuf->type != TB_GCHAT) return;

	x = iks_make_pres(IKS_TYPE_SET, IKS_SHOW_UNAVAILABLE, iks_id_printx(curbuf->id, IKS_ID_USER | IKS_ID_SERVER), NULL);
	iks_send(net.parser, x);
	iks_delete(x);

	ui_buffer_delete(curbuf);
}


void gchat_send(textbuf *tb, char *msg)
{
	iks *x;

	x = iks_make_msg(IKS_TYPE_GROUPCHAT, iks_id_printx(tb->id, IKS_ID_USER | IKS_ID_SERVER), msg, NULL);
	iks_send(net.parser, x);
	iks_delete(x);

}


void gchat_msg(ikspak *pak)
{
        textbuf *tb;
  
        tb = target_textbuf(pak);

        if (!tb) {
                /* invite? */
                 return;
        }

	if(pak->from->resource)
		ui_print(tb, "<%s> %s\n", pak->from->resource, iks_find_cdata(pak->x, "body"));
	else
		ui_print(tb, "*** %s\n", iks_find_cdata(pak->x, "body"));

}

void gchat_whisper(textbuf *tb, ikspak *pak)
{
	ui_print(tb, "<%s -> %s> %s\n", pak->from->resource, tb->id->resource, iks_find_cdata(pak->x, "body"));
        ui_set_tab_content("/w %s", pak->from->resource);
}
        
textbuf *target_textbuf(ikspak *pak) 
{
        textbuf *tb;

	for(tb = list_first(&textbufs); tb; tb = list_next(tb))
	{
		if(iks_id_cmpx(pak->from, tb->id, IKS_ID_USER | IKS_ID_SERVER) == 0)
                         return tb;
	}

        return NULL;
}

void cmd_whisper(char *arg) 
{
        iks *x;
        iksid *w;
        char *tokarg = strdup(arg);
        char *to, *body;

	if(curbuf->type != TB_GCHAT)
        {
                  goto usage;
        }

        if(!arg || *arg == '\0') {
                  goto usage;
        }

        to = strtok(tokarg, " ");
        body = strtok(NULL, "");

        if(!to || !body) {
                  goto usage;
        }

        w = iks_id_new(NULL, iks_id_printx(curbuf->id, IKS_ID_USER | IKS_ID_SERVER));
        w->resource = to;

        x = iks_make_msg(IKS_TYPE_CHAT, iks_id_print(w), body, NULL);
	iks_send(net.parser, x);
	iks_delete(x);

	ui_print(curbuf, "<%s -> %s> %s\n", curbuf->id->resource, w->resource, body);
        ui_set_tab_content("/w %s", w->resource);
        goto cleanup;

 usage:
        gui_attr(GA_NOTICE);
        gui_print("Usage: whisper <nick> <message>\n");
        gui_print("[only works in chat rooms]\n");

 cleanup:
        free(tokarg);
}

int gchat_pres(ikspak *pak)
{
	textbuf *tb;
	char *tmp;

	tb = list_first(&textbufs);
	while(tb)
	{
		if(iks_id_cmpx(pak->from, tb->id, IKS_ID_USER | IKS_ID_SERVER) == 0)
			break;
		tb = list_next(tb);
	}
	if(!tb) return(0);

	switch(pak->show)
	{
	case IKS_SHOW_UNAVAILABLE: tmp = "unavailable."; break;
	case IKS_SHOW_AVAILABLE: tmp = "available."; break;
	case IKS_SHOW_AWAY: tmp = "away."; break;
	case IKS_SHOW_XA: tmp = "xa."; break;
	case IKS_SHOW_DND: tmp = "dnd."; break;
	case IKS_SHOW_CHAT: tmp = "ready for chat."; break;
	default: tmp = "(unknown status)"; break;
	}

	ui_print(tb, "*** %s is %s\n", pak->from->resource, tmp);

	return 1;
}
