/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** perl scripting
*/

#include "common.h"

#undef PACKAGE
// #undef bool  this makes it incompileable for me, perhaps the next resolves the warnings:
#ifndef HAS_BOOL
#ifdef bool
#define HAS_BOOL
#endif
#endif
#include <EXTERN.h>
#include <perl.h>
#include <XSUB.h>
#undef PACKAGE

static PerlInterpreter *pl_perl = NULL;
static char *pl_msg_handler = NULL;
static char *pl_prs_handler = NULL;

SV *pl_execute(char *func, char *args);
int pl_execute_file(char *fname);

XS(XS_info);
XS(XS_print);
XS(XS_add_message_handler);
XS(XS_add_presence_handler);
XS(XS_send_message);


void cmd_perl(char *arg)
{
	int ret;

	if(!arg) return;

	ret = pl_execute_file(arg);
	switch(ret)
	{
	case 0:
		gui_print("Script loaded.\n");
		break;
	case 1:
		gui_print("Error in script.\n");
		break;
	case 2:
		gui_print("Cannot find script file '%s'.\n", arg);
	}
      if (!options.compact_lines)
         gui_print ("\n");
}


void pl_handle_message(char *from, char *body)
{
	char *tmp;

	if(!pl_msg_handler) return;

	tmp = malloc(strlen(from) + strlen(body) + 10);
	sprintf(tmp, "%s %s", from, body);
	pl_execute(pl_msg_handler, tmp);
	free(tmp);
}


void pl_setup(void)
{
	char *args[] = { "", "-e", "0" };
	char load_file[] = 
"sub load_file()\n"
"{\n"
"  (my $file_name) = @_;\n"
"  open FH, $file_name or return 2;\n"
"  local($/) = undef;\n"
"  $file = <FH>;\n"
"  close FH;\n"
"  eval $file;\n"
"  eval $file if $@;\n"
"  return 1 if $@;\n"
"  return 0;\n"
"}";

	pl_perl = perl_alloc();
	perl_construct(pl_perl);

	perl_parse(pl_perl, NULL, 3, args, NULL);
	perl_eval_pv(load_file, TRUE);

	newXS("JabberX::info", XS_info, "JabberX");
	newXS("JabberX::print", XS_print, "JabberX");
	newXS("JabberX::add_message_handler", XS_add_message_handler, "JabberX");
	newXS("JabberX::add_presence_handler", XS_add_presence_handler, "JabberX");
	newXS("JabberX::send_message", XS_send_message, "JabberX");
}


void pl_cleanup(void)
{
	perl_destruct(pl_perl);
	perl_free(pl_perl);
	pl_perl = NULL;
}


SV *pl_execute(char *func, char *args)
{
	static char *tmp = NULL;

	if(tmp) free(tmp);
	tmp = malloc(strlen(func) + strlen(args) + 10);
	sprintf(tmp, "&%s('%s')", func, args);
	return perl_eval_pv(tmp, TRUE);
}


int pl_execute_file(char *fname)
{
	SV *tmp;
	tmp = pl_execute("load_file", fname);
	return SvNV(tmp);
}


/*
** our perl functions
*/

XS(XS_info)
{
	int tmp;
	dXSARGS;

	switch(atoi(SvPV(ST(0), tmp)))
	{
	case 0:
		XST_mPV(0, VERSION);
		break;

	case 1:
		XST_mPV(0, iks_id_print(net.id));
		break;

	}
	XSRETURN(1);
}


XS(XS_print)
{
	int i, tmp;
	char *out;
	dXSARGS;

	for(i=0; i<items; ++i)
	{
		out = SvPV(ST(i), tmp);
		gui_print("Perl says: %s\n", out);
	}

	XSRETURN_EMPTY;
}


XS(XS_add_message_handler)
{
	int tmp;
	dXSARGS;

	pl_msg_handler = strdup(SvPV(ST(0), tmp));

	XSRETURN_EMPTY;
}


XS(XS_add_presence_handler)
{
	int tmp;
	dXSARGS;

	pl_prs_handler = strdup(SvPV(ST(0), tmp));

	XSRETURN_EMPTY;
}


XS(XS_send_message)
{
	iks *x;
	char *to, *body;
	int tmp;
	dXSARGS;

	to = SvPV(ST(0), tmp);
	body = SvPV(ST(1), tmp);

	x = iks_make_msg(IKS_TYPE_NONE, to, body, NULL);
	iks_send(net.parser, x);
	iks_delete(x);

	XSRETURN_EMPTY;
}
