/*
** JabberX (Jabber Client)
** Copyright (c) 2002 Evan Prodromou <misterbad@users.sourceforge.net>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** searching
*/

#include "common.h"

void show_search_fields(ikspak *pak);
void show_search_results(ikspak *pak);

void cmd_search(char *arg)
{
        iks *x, *q;
        char *tokarg = strdup(arg);
        char *to;
        char *fv, *field, *value;
        char *eq;

	x = iks_make_iq(IKS_TYPE_SET, IKS_NS_SEARCH);
        q = iks_find(x, "query");

	if(arg && strlen(arg))
        {
                
                to = strtok(tokarg, " ");

                if (!to)
                          goto usage;
                else 
                {
                          iks_insert_attrib(x, "to", to);
                          
                          while (fv = strtok(NULL, " "))
                          {
                                   eq = strchr(fv, '=');
                                   if (!eq)
                                         goto usage;
                                   else
                                   {
                                         *eq = 0;
                                         iks_insert_cdata(iks_insert(q, fv), eq + 1, -1);
                                   }
                          }

                          iks_send(net.parser, x);
                          goto cleanup;
                }
        }

usage:
        gui_attr(GA_NOTICE);
        gui_print("Usage: search <jid> <field>=<value> <field>=<value> ...\n");
        if (!options.compact_lines)
                gui_print ("\n");
        
cleanup:
	iks_delete(x);
        free(tokarg);
}

void cmd_fields(char *arg)
{
        iks *x;

	if(arg && strlen(arg))
        {
              x = iks_make_iq(IKS_TYPE_GET, IKS_NS_SEARCH);
              iks_insert_attrib(x, "to", arg);
              iks_send(net.parser, x);
              iks_delete(x);
        }
        else
        {
              gui_attr(GA_NOTICE);
              gui_print("Usage: fields <jid>\n");
              if (!options.compact_lines)
                gui_print ("\n");
        }
}

void search_parse(ikspak *pak)
{
	iks *x;

	x = iks_find(pak->x, "query");

        if (iks_find(x, "key") || iks_find(x, "instructions"))
                 show_search_fields(pak);
        else
                 show_search_results(pak);
}

void show_search_fields(ikspak *pak) {

        iks *q, *x, *i;
        char *name;

	gui_attr(GA_NOTICE);

	gui_ts_print("Search fields from '%s':\n", iks_id_print(pak->from));
	q = iks_find(pak->x, "query");

	for (x = iks_child(q); x; x = iks_next(x))
	{
		if(iks_type(x) == IKS_TAG)
		{
                         name = iks_name(x);
                         if (iks_strcmp(name, "key") &&
                             iks_strcmp(name, "instructions"))
                         {
                               gui_print("-> %s\n", iks_name(x));
                         }
		}
	}

        i = iks_find(q, "instructions");

        if (i) 
        {
                gui_print("Search instructions:\n");
                gui_print(iks_cdata(iks_child(i)));
                gui_print("\n");
        }

        if (!options.compact_lines)
               gui_print ("\n");
}

void show_search_results(ikspak *pak) {
        iks *q, *x, *y;
        char *name;
        int trunc = 0;

	gui_attr(GA_NOTICE);

	gui_ts_print("Search results from '%s':\n", iks_id_print(pak->from));
	q = iks_find(pak->x, "query");

	for (x = iks_child(q); x; x = iks_next(x))
	{
		if(iks_type(x) == IKS_TAG)
		{
                         name = iks_name(x);
                         if (!iks_strcmp(name, "item"))
                         {
                               gui_print("-> %s:\n", iks_find_attrib(x, "jid"));

                               for (y = iks_child(x); y; y = iks_next(y)) {
                                         if(iks_type(y) == IKS_TAG)
                                                  gui_print("--> %s: %s\n", iks_name(y), iks_cdata(iks_child(y)));
                               }
                         }
                         else if (!iks_strcmp(name, "truncated")) {
                               trunc = 1;
                         }
		}
	}

        if (trunc)
                gui_print("Search results truncated; too many results.\n");

        if (!options.compact_lines)
               gui_print ("\n");
}

void search_error(ikspak *pak)
{
         iks *x;

         gui_attr(GA_NOTICE);
         gui_ts_print("Search result from '%s':\n", iks_id_print(pak->from));
         x = iks_find(pak->x, "error");
         if(x)
         {
           gui_print("Error %s: %s.\n", iks_find_attrib(x, "code"), iks_cdata(iks_child(x)));
           if (!options.compact_lines)
             gui_print ("\n");
         }
}

